package metrotest;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Dispatch;
import javax.xml.ws.Service;
import javax.xml.ws.Service.Mode;

import org.eclipse.jetty.server.Connector;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.nio.SelectChannelConnector;
import org.eclipse.jetty.util.thread.QueuedThreadPool;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import com.sun.xml.ws.api.SOAPVersion;
import com.sun.xml.ws.api.message.Message;
import com.sun.xml.ws.api.message.Messages;

public class TestCdata
{
	public static void main(String[] args) throws Exception 
	{
		Server server = createJettyServer(8081);
		
		URL wsdlURL;
		try
		{
			wsdlURL = new File("test.wsdl").toURI().toURL();
		}
		catch(MalformedURLException e)
		{
			throw new RuntimeException("Invalid URL ", e);
		}
		
		Service service = Service.create(wsdlURL, new QName("urn:qualityAssurance", "qualityAssuranceService"));
		QName portName = new QName("urn:qualityAssurance", "qualityAssuranceServicePort");
		Dispatch<Message> disp = service.createDispatch(portName, Message.class, Mode.MESSAGE);
		setRequestContextValues(disp.getRequestContext(), "http://localhost:8081/", "urn:qualityAssurancegetTests");
		
		String body = "<?xml version='1.0' encoding='utf-8'?><soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"><soapenv:Body></soapenv:Body></soapenv:Envelope>";
		InputStream bodyStream = new ByteArrayInputStream(body.getBytes());
		Message request = Messages.create(new StreamSource(bodyStream), SOAPVersion.SOAP_11);
		Message response = disp.invoke(request);
		
		SAXSource saxSourceResponse = (SAXSource) response.readPayloadAsSource();
		
		importXmlStream(saxSourceResponse);
		
		server.stop();
	}
	
	private static void setRequestContextValues(Map<String, Object> requestContext,	String postLocation, String soapAction)
	{
		requestContext.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, postLocation);
		requestContext.put(BindingProvider.SOAPACTION_USE_PROPERTY, true);
		requestContext.put(BindingProvider.SOAPACTION_URI_PROPERTY, soapAction);
	}
	
	private static void importXmlStream(SAXSource saxSource) throws IOException, SAXException 
	{
		XMLTestStreamingHandler handler = new XMLTestStreamingHandler();
		XMLReader reader = saxSource.getXMLReader();
		InputSource inputSource = saxSource.getInputSource();
		reader.setContentHandler(handler);
		reader.parse(inputSource);
	}
	
	public static Server createJettyServer(int runtimePort) throws Exception
	{
		Server jettyServer = new Server();
		Connector connector = new SelectChannelConnector();
		connector.setPort(runtimePort);
		jettyServer.addConnector(connector);
		jettyServer.setSendDateHeader(true);
		jettyServer.setThreadPool(new QueuedThreadPool());
		jettyServer.setHandler(new MyHandler());
		jettyServer.start();
		return jettyServer;
	}
}